#ifndef ARCH_CC1010_PHAL_H
#define ARCH_CC1010_PHAL_H

#include <chipcon/inc/Chipcon/hal.h>

/** @brief Enable global interrupts
 *
 */
#define phalGlobalInterruptEnable()    (EA=1)

/** @brief Disable global interrupts
 *
 */
#define phalGlobalInterruptDisable()   (EA=0)

/** @brief Enter critical section
 *
 * This is different from disabling interrupts globally
 * as it remembers whether interrupts where enabled or disabled
 * at the time of entering the critical section and restores the
 * correct value at leaving (with halExitCriticalSection)
 *
 * Actually it is not even necessary that this mechanism is
 * implemented via disabling interrupts, so don't rely on it.
 *
 * phalEnterCriticalSection() and phalExitCriticalSection must match:
 *
 * valid:
 *         phalEnterCriticalSection();
 *         ...
 *         phalExitCriticalSection();
 *
 * invalid:
 *         if( x == 5 )
 *             phalEnterCriticalSection();
 * 	       else
 *             doNothing();
 *         ...
 *         phalExitCriticalSection;
 *
 * @see halExitCriticalSection
 */
#ifdef SDCC
#define phalEnterCriticalSection() do { __critical {
#else
void phalEnterCriticalSection();
#endif

/** @brief Exit critical section
 *
 * @see halEnterCriticalSection
 */
#ifdef SDCC
#define phalExitCriticalSection() } while(0)
#else
void phalExitCriticalSection();
#endif

/** @brief Init chip on which this code runs
 *
 * The actual behaviour might change from architecture to architecture.
 */
void phalChipInit(uint8_t parameter);

/** @brief Set the direction of a port to incoming or outgoing
 *
 * @param portNum	port number to be set
 * @param bitNum	bit (pin) of the port to be set
 * @param direction
 */
void phalSetPortBitDir(uint8_t portNum, uint8_t bitNum, uint8_t direction);

void phalTimer0Init(uint8_t option);
void phalTimer0Start();
void phalTimer0Stop();
uint8_t phalTimer0Overflow();
void phalTimer0ClearOverflow();
uint16_t phalTimer0Get();
void phalTimer0Set(uint16_t value);
void phalTimer2Init();
void phalTimer2Start();
void phalTimer2Stop();
uint8_t phalTimer2Overflow();
void phalTimer2ClearOverflow();
void phalTimer3Init(byte option);

/**
 * Delay
 */
void phalDelayMs(uint8_t milliseconds);
void phalDelayUs(uint8_t microseconds);

/**
 * Uart
 */
void phalUart0Init( uint8_t mode );
void phalUart0ReceiveOn();
void phalUart0ReceiveOff();
void phalUart0SendByte( uint8_t datum );
uint8_t phalUart0ByteSent();
uint8_t phalUart0ReceiveBytePolled();
uint8_t phalUart0CheckForNewData();
void phalUart1Init();
void phalUart1ReceiveOn();
void phalUart1ReceiveOff();
void phalUart1SendByte( uint8_t datum );
uint8_t phalUart1ByteSent();
void phalUart1SendBytePolledEscaped( uint8_t datum );
uint8_t phalUart1ReceiveBytePolled();
uint8_t phalUart1CheckForNewData();
void phalUart1ClearReceiveFlag();

/**
 * ADC
 */
void phalADCInit();
void phalADCEnable();
void phalADCDisable();
void phalADCSetVDDReference();
void phalADCSetInternalReference();
void phalADCSetChannel(uint8_t channel);
uint8_t phalADCRead8();
uint16_t phalADCRead10();

/**
 * SPI
 */
void phalSPIEnable();
void phalSPIDisable();


/**
 * Clock
 */
void phalClockInit();
void phalUpdateClock();
uint16_t phalClockGetInterval( uint16_t start_time );
uint8_t phalClockGetLittleInterval( uint8_t start_little_time );


/**
 * ID
 */
 #define	CC_MY_ID_CHECK					103
#define CC_RAM_FOR_ROM_BUFFER_ADDRESS	0x80
#define CC_ROM_ADDRESS					0x7f80
#define CC_ROM_MY_ID_POSITION			0
#define CC_ROM_MY_ID_CHECK_POSITION		4

#define CC_DEFAULT_ID_0  100
#define CC_DEFAULT_ID_1  100
#define CC_DEFAULT_ID_2  100
#define CC_DEFAULT_ID_3  100
 
uint8_t phalGetID();
void phalWriteID( uint8_t* id );
void phalEraseID();
void phalCheckID();

#endif /* ARCH_CC1010_PHAL_H */
