/*
 * This is the buffer management code file template.
 * Define BUFFER_PREFIX, BUFFER_LINES and BUFFER_SIZE_PER_LINE
 * and then include it.
 * 
 * BUFFER_PREFIX   the prefix that will be expanded before any
 *                 function name or variable name to make it unique
 *                 
 * BUFFER_LINES    number of buffer lines to allocate
 * 
 * BUFFER_SIZE_PER_LINE  number of bytes per buffer line
 */
#include INC_ARCH(phal.h)

/* BUFFER_PREFIX, BUFFER_LINES and BUFFER_SIZE_PER_LINE have to be defined! */ 
#ifndef BUFFER_PREFIX
	#error You have to define BUFFER_PREFIX
#endif
#ifndef BUFFER_LINES
	#error You have to define BUFFER_LINES
#endif
#ifndef BUFFER_SIZE_PER_LINE
	#error You have to define BUFFER_SIZE_PER_LINE
#endif

/* 
 * undefine macros in case they have already been defined
 * by another inclusion of this template
 */
#undef ADD_PREFIX
#undef PREFIX_CONCAT1
#undef PREFIX_CONCAT2

/* 
 * concatenation macros
 * need indirection over PREFIX_CONCAT1() so that BUFFER_PREFIX will
 * be expanded correctly
 */
#define ADD_PREFIX(a) PREFIX_CONCAT1(BUFFER_PREFIX,a)
#define PREFIX_CONCAT1(a,b) PREFIX_CONCAT2(a,b)
#define PREFIX_CONCAT2(a,b) a##b

/* the buffer */
uint8_t XRAM ADD_PREFIX(_buffer)[BUFFER_LINES][BUFFER_SIZE_PER_LINE];
/* buffer management */
ADD_PREFIX( _buffer_management_type ) XRAM ADD_PREFIX(_buffer_management)[BUFFER_LINES];


/**@brief buffer allocate template
 * 
 * @author Martin Buml <baeuml@teco.edu>
 * 
 * This will allocate a free buffer line and mark it allocated.
 * 
 * Unless your compiler supports critical functions (like SDCC),
 * make sure that allocate is not called within an interrupt,
 * because it
 *   1. is not reentrant (no overlay protection)
 *   2. there is an unhandled race condition, which might lead
 *      to the situation, that a buffer line is allocated more
 *      than once to different "threads". This is definitely 
 *      something you want to avoid!
 * 
 *  
 * Usage:
 *      If your BUFFER_PREFIX is for example TCL, then:
 * 
 *      handle = TCLAllocate();
 * 		if( handle != 255 )
 * 
 * @return index of allocated line in buffer, if successful
 *         255 if there was no free buffer available
 */
uint8_t ADD_PREFIX(Allocate)() CRITICALFUNCTION
{
	uint8_t i;
	
	for( i = 0; i < BUFFER_LINES; ++i )
	{
		/* iterate over all entries, return the first free one
		 * this might not be the fastest way to do this, but
		 * it is simple and speed will be ok with small buffer sizes
		 */
		if( ADD_PREFIX(_buffer_management)[i].allocated == 0 )
		{
			ADD_PREFIX(_buffer_management)[i].allocated == 1;
			return i;
		}
	}
	
	/* return that there was no free buffer available */
	return 255;
}

/**@brief buffer release template
 * 
 * @author Martin Buml <baeuml@teco.edu>
 * 
 * This will release a buffer line and mark it free again.
 * 
 * Usage:
 *      If your BUFFER_PREFIX is for example TCL, then:
 * 
 *      TCLRelease( handle );
 * 
 * @param handle the index of the buffer line to be released
 */
void ADD_PREFIX(Release)( uint8_t handle )
{
	/* check that parameter is valid */
	if( handle >= BUFFER_LINES ) return;
	
	/* release buffer line */
	ADD_PREFIX(_buffer_management)[handle].allocated = 0;
}

